/*
 Gyro X, Y, Z Demo
 Version: 1.0
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 3/10/2021
 Last Modified: 3/10/2021
 
 */


#include <gba_console.h>
#include <gba_video.h>
#include <gba_interrupt.h>
#include <gba_systemcalls.h>
#include <gba_input.h>
#include <gba_timers.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define GBAROM ((u16*)0x08000000)
#define CART_SRAM 0xe000000

// Thanks to djedditt for these
#define GPIO_PORT_DATA		(*(volatile unsigned short*)0x80000C4)
#define GPIO_PORT_DIRECTION	(*(volatile unsigned short*)0x80000C6)
#define GPIO_PORT_CONTROL	(*(volatile unsigned short*)0x80000C8)


int16_t read_gyro(void) {
	uint16_t gyroReg = 0;
	
	// CS low
	GPIO_PORT_DATA = 0x02;
	
	// Read data
	for (int8_t x = 15; x >= 0; x--) { 
		// Clk low
		GPIO_PORT_DATA = 0x00;
		
		for (u32 volatile x = 0; x < 50; x++) {}
		
		// Clk high
		GPIO_PORT_DATA = 0x02;
		
		for (u32 volatile x = 0; x < 50; x++) {}
		
		// Read rom
		u8 bitbuffer[2];
		memcpy(bitbuffer, (void*)((int) ((u16*)0x080000C4)), 1); 
		
		if (bitbuffer[0] & 0x04) {
			gyroReg |= (1<<x);
		}
		//iprintf("\x1b[%i;0H %i   ", line, bitbuffer[0]);
		//line++;
	}
	GPIO_PORT_DATA = 0x03;
	
	return gyroReg;
}

int main(void) {
	irqInit();
	irqEnable(IRQ_VBLANK);
	
	consoleInit(	0,		// charbase
					4,		// mapbase
					0,		// background number
					NULL,	// font
					0, 		// font size
					15		// 16 color palette
	);
	
	
	// set the screen colors, color 0 is the background color
	// the foreground color is index 1 of the selected 16 color palette
	BG_COLORS[0]=RGB8(64,64,64);
	BG_COLORS[241]=RGB5(31,31,31);
	SetMode(MODE_0 | BG0_ON);
	
	iprintf("\x1b[0;8HinsideGadgets");
	iprintf("\x1b[1;4HGyro X, Y, Z Demo v1.0");
	
	GPIO_PORT_CONTROL = 0x01;
	GPIO_PORT_DIRECTION = 0x03;
	GPIO_PORT_DATA = 0x03;
	
	int16_t gyroZWW = 0; // Warioware value
	int16_t gyroX = 0;
	int16_t gyroY = 0;
	int16_t gyroZ = 0;
	
	int8_t dotXLocation = 14;
	int8_t dotYLocation = 10;
	int8_t previousdotXLocation = 14;
	int8_t previousdotYLocation = 10;
	
	while (1) {
		// Read gyro
		gyroZWW = read_gyro() >> 1;
		gyroX = (read_gyro() / 30); // /30 to decrease sensitivity
		gyroY = (read_gyro() / 30);
		gyroZ = (read_gyro() / 30);
		
		iprintf("\x1b[3;0HGyro Z WW = %i     ", gyroZWW);
		iprintf("\x1b[4;0HGyro X = %i   ", gyroX);
		iprintf("\x1b[5;0HGyro Y = %i    ", gyroY);
		iprintf("\x1b[6;0HGyro Z = %i    ", gyroZ);
		
		//for (u32 volatile x = 0; x < 5000; x++) {} // Delay a tiny bit
		
		// Basic movement detection, this could be done much better
		if (gyroX >= 150) {
			dotYLocation -= 1;
		}
		if (gyroX <= -150) {
			dotYLocation += 1;
		}
		if (gyroY >= 150) {
			dotXLocation += 1;
		}
		if (gyroY <= -150) {
			dotXLocation -= 1;
		}
		
		iprintf("\x1b[%i;%iH ", previousdotYLocation, previousdotXLocation);
		iprintf("\x1b[%i;%iH.", dotYLocation, dotXLocation);
		
		previousdotXLocation = dotXLocation;
		previousdotYLocation = dotYLocation;
		
		//VBlankIntrWait();
	}
}
